package starj.util.xml;

import java.io.*;

import starj.util.StringUtils;

public class XMLPrintStream extends PrintStream {
    public static final int DEFAULT_INDENT_STEP = 4;
    private int indent_step;
    private String indent_string = "";
    private int level = 0;
    private boolean need_indent = true;

    public XMLPrintStream(OutputStream out) {
        super(out);
        this.indent_step = DEFAULT_INDENT_STEP;
    }

    public XMLPrintStream(OutputStream out, int indentationStep) {
        super(out);
        this.indent_step = indentationStep;
    }


    public XMLPrintStream(OutputStream out, boolean autoFlush) {
        super(out, autoFlush);
        this.indent_step = DEFAULT_INDENT_STEP;
    }
    

    public XMLPrintStream(OutputStream out, boolean autoFlush,
            int indentationStep) {
        super(out, autoFlush);
        this.indent_step = indentationStep;
    }

    public static String escape(String s) {
        StringBuffer sb = new StringBuffer(s);
        for (int i = 0; i < sb.length(); i++) {
            char c = sb.charAt(i);
            switch (c) {
                case '<':
                    sb.replace(i, i + 1, "&lt;");
                    break;
                case '>':
                    sb.replace(i, i + 1, "&gt;");
                    break;
                case '\"':
                    sb.replace(i, i + 1, "&quot;");
                    break;
                case '\'':
                    sb.replace(i, i + 1, "&apos;");
                    break;
                case '&':
                    sb.replace(i, i + 1, "&amp;");
                    break;
                default:
                    break;
            }
        }

        return sb.toString();
    }

    private String make_open_tag(String name, XMLAttribute[] attribs) {
        String rv = "<" + name;
        if (attribs != null) {
            for (int i = 0; i < attribs.length; i++) {
                rv += " " + attribs[i];
            }
        }
        return rv + ">";
    }

    private String make_open_tag(String name, XMLAttributes attribs) {
        return this.make_open_tag(name, attribs.toArray());
    }

    private String make_close_tag(String name) {
        return "</" + name + ">";
    }

    private String make_standalone_tag(String name, XMLAttribute[] attribs) {
        String rv = "<" + name;
        if (attribs != null) {
            for (int i = 0; i < attribs.length; i++) {
                rv += " " + attribs[i];
            }
        }
        return rv + "/>";
    }

    private String make_standalone_tag(String name, XMLAttributes attribs) {
        return this.make_standalone_tag(name, attribs.toArray());
    }

    public void openln(String tag) {
        this.openln(tag, (XMLAttribute[]) null);
    }

    public void openln(String tag, XMLAttribute attrib) {
        XMLAttribute[] attribs = new XMLAttribute[1];
        attribs[0] = attrib;
        this.openln(tag, attribs);
    }

    public void openln(String tag, XMLAttribute[] attribs) {
        this.println(this.make_open_tag(tag, attribs));
        this.indent();
    }
    
    public void openln(String tag, XMLAttributes attribs) {
        this.openln(tag, attribs.toArray());
    }

    public void closeln(String tag) {
        this.unindent();
        this.println(this.make_close_tag(tag));
    }

    public void open(String tag) {
        this.open(tag, (XMLAttribute[]) null);
    }

    public void open(String tag, XMLAttribute attrib) {
        XMLAttribute[] attribs = new XMLAttribute[1];
        attribs[0] = attrib;
        this.open(tag, attribs);
    }

    public void open(String tag, XMLAttribute[] attribs) {
        this.print(this.make_open_tag(tag, attribs));
    }
    
    public void open(String tag, XMLAttributes attribs) {
        this.open(tag, attribs.toArray());
    }

    public void close(String tag) {
        this.print(this.make_close_tag(tag));
    }

    private void print_indent() {
        if (this.need_indent) {
            super.print(this.indent_string);
            this.need_indent = false;
        }
    }
    
    private void indent() {
        this.level += this.indent_step;
        this.indent_string = StringUtils.indent("", this.level);
    }

    private void unindent() {
        this.level -= this.indent_step;
        if (this.level < 0) {
            this.level = 0;
        }
        this.indent_string = this.indent_string.substring(0, this.level);
    }

    /* Standard PrintStream methods (with added indentation support) */

    public void print(boolean b) {
        this.print_indent();
        super.print(b);
    }

    public void print(char c) {
        this.print_indent();
        super.print(c);
    }

    public void print(char[] s) {
        this.print_indent();
        super.print(s);
    }
  
    public void print(double d) {
        this.print_indent();
        super.print(d);
    }

    public void print(float f) {
        this.print_indent();
        super.print(f);
    }

    public void print(int i) {
        this.print_indent();
        super.print(i);
    }

    public void print(long l) {
        this.print_indent();
        super.print(l);
    }

    public void print(Object obj) {
        this.print(String.valueOf(obj));
        
    }
   
    public void print(String s) {
        this.print_indent();
        super.print(s);
        if (s.endsWith("\n")) {
            this.need_indent = true;
        }
    }

    public void println() {
        super.println();
        this.need_indent = true;
    }


    public void println(boolean x) {
        this.print_indent();
        super.println(x);
        this.need_indent = true;
    }

    public void println(char x) {
        this.print_indent();
        super.println(x);
        this.need_indent = true;
    }

    public void println(char[] x) {
        this.print_indent();
        super.println(x);
        this.need_indent = true;
    }

    public void println(double x) {
        this.print_indent();
        super.println(x);
        this.need_indent = true;
    }

    public void println(float x) {
        this.print_indent();
        super.println(x);
        this.need_indent = true;
    }
    
    public void println(int x) {
        this.print_indent();
        super.println(x);
        this.need_indent = true;
    }

    public void println(long x) {
        this.print_indent();
        super.println(x);
        this.need_indent = true;
    }

    public void println(Object x) {
        this.print_indent();
        super.println(x);
        this.need_indent = true;
    }

    public void println(String x) {
        this.print_indent();
        super.println(x);
        this.need_indent = true;
    }

    public void tag(String tag) {
        this.print(this.make_standalone_tag(tag, (XMLAttribute[]) null));
    }

    public void tag(String tag, boolean x) {
        this.open(tag);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, char x) {
        this.open(tag);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, char[] x) {
        this.open(tag);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, double x) {
        this.open(tag);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, float x) {
        this.open(tag);
        this.print(x);
        this.close(tag);
    }
    
    public void tag(String tag, int x) {
        this.open(tag);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, long x) {
        this.open(tag);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, Object x) {
        this.open(tag);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, String x) {
        this.open(tag);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute attrib, boolean x) {
        this.open(tag, attrib);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute attrib, char x) {
        this.open(tag, attrib);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute attrib, char[] x) {
        this.open(tag, attrib);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute attrib, double x) {
        this.open(tag, attrib);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute attrib, float x) {
        this.open(tag, attrib);
        this.print(x);
        this.close(tag);
    }
    
    public void tag(String tag, XMLAttribute attrib, int x) {
        this.open(tag, attrib);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute attrib, long x) {
        this.open(tag, attrib);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute attrib, Object x) {
        this.open(tag, attrib);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute attrib, String x) {
        this.open(tag, attrib);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute[] attribs, boolean x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute[] attribs, char x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute[] attribs, char[] x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute[] attribs, double x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute[] attribs, float x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }
    
    public void tag(String tag, XMLAttribute[] attribs, int x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute[] attribs, long x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute[] attribs, Object x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttribute[] attribs, String x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttributes attribs, boolean x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttributes attribs, char x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttributes attribs, char[] x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttributes attribs, double x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttributes attribs, float x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }
    
    public void tag(String tag, XMLAttributes attribs, int x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttributes attribs, long x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttributes attribs, Object x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tag(String tag, XMLAttributes attribs, String x) {
        this.open(tag, attribs);
        this.print(x);
        this.close(tag);
    }

    public void tagln(String tag) {
        this.tag(tag);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, boolean x) {
        this.tag(tag, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, char x) {
        this.tag(tag, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, char[] x) {
        this.tag(tag, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, double x) {
        this.tag(tag, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, float x) {
        this.tag(tag, x);
        super.println();
        this.need_indent = true;
    }
    
    public void tagln(String tag, int x) {
        this.tag(tag, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, long x) {
        this.tag(tag, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, Object x) {
        this.tag(tag, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, String x) {
        this.tag(tag, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute attrib) {
        this.tag(tag, attrib);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute attrib, boolean x) {
        this.tag(tag, attrib, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute attrib, char x) {
        this.tag(tag, attrib, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute attrib, char[] x) {
        this.tag(tag, attrib, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute attrib, double x) {
        this.tag(tag, attrib, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute attrib, float x) {
        this.tag(tag, attrib, x);
        super.println();
        this.need_indent = true;
    }
    
    public void tagln(String tag, XMLAttribute attrib, int x) {
        this.tag(tag, attrib, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute attrib, long x) {
        this.tag(tag, attrib, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute attrib, Object x) {
        this.tag(tag, attrib, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute attrib, String x) {
        this.tag(tag, attrib, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute[] attribs) {
        this.tag(tag, attribs);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute[] attribs, boolean x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute[] attribs, char x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute[] attribs, char[] x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute[] attribs, double x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute[] attribs, float x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }
    
    public void tagln(String tag, XMLAttribute[] attribs, int x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute[] attribs, long x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute[] attribs, Object x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttribute[] attribs, String x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttributes attribs) {
        this.tag(tag, attribs);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttributes attribs, boolean x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttributes attribs, char x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttributes attribs, char[] x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttributes attribs, double x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttributes attribs, float x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }
    
    public void tagln(String tag, XMLAttributes attribs, int x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttributes attribs, long x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttributes attribs, Object x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }

    public void tagln(String tag, XMLAttributes attribs, String x) {
        this.tag(tag, attribs, x);
        super.println();
        this.need_indent = true;
    }
}
